@@ -1,7 +1,7 @@
 require 'puppet/version'
 
-if Gem::Version.new(RUBY_VERSION) < Gem::Version.new("1.9.3")
-  raise LoadError, "Puppet #{Puppet.version} requires ruby 1.9.3 or greater."
+if Gem::Version.new(RUBY_VERSION.dup) < Gem::Version.new("1.9.3")
+  raise LoadError, _("Puppet %{version} requires ruby 1.9.3 or greater.") % { version: Puppet.version }
 end
 
 Puppet::OLDEST_RECOMMENDED_RUBY_VERSION = '2.1.0'
@@ -17,6 +17,7 @@ require 'puppet/settings'
 require 'puppet/util/feature'
 require 'puppet/util/suidmanager'
 require 'puppet/util/run_mode'
+# PSON is deprecated, use JSON instead
 require 'puppet/external/pson/common'
 require 'puppet/external/pson/version'
 require 'puppet/external/pson/pure'
@@ -27,14 +28,46 @@ require 'puppet/external/pson/pure'
 begin
   require 'gettext-setup'
   require 'locale'
+
+  # e.g. ~/code/puppet/locales. Also when running as a gem.
+  local_locale_path = File.absolute_path('../locales', File.dirname(__FILE__))
+  # e.g. /opt/puppetlabs/puppet/share/locale
+  posix_system_locale_path = File.absolute_path('../../../share/locale', File.dirname(__FILE__))
+  # e.g. C:\Program Files\Puppet Labs\Puppet\puppet\share\locale
+  win32_system_locale_path = File.absolute_path('../../../../../puppet/share/locale', File.dirname(__FILE__))
+
+  if File.exist?(local_locale_path)
+    locale_path = local_locale_path
+  elsif Puppet::Util::Platform.windows? && File.exist?(win32_system_locale_path)
+    locale_path = win32_system_locale_path
+  elsif !Puppet::Util::Platform.windows? && File.exist?(posix_system_locale_path)
+    locale_path = posix_system_locale_path
+  else
+    # We couldn't load our locale data.
+    raise LoadError, "could not find locale data, skipping Gettext initialization"
+  end
+
+  Puppet::LOCALE_PATH = locale_path
   Puppet::GETTEXT_AVAILABLE = true
 rescue LoadError
   def _(msg)
     msg
   end
+
+  def n_(*args, &block)
+    # assume two string args (singular and plural English form) and the count
+    # to pluralize on
+    plural = args[2] == 1 ? args[0] : args[1]
+    # if a block is passed, prefer that over the string selection above
+    block ? block.call : plural
+  end
+
+  Puppet::LOCALE_PATH = nil
   Puppet::GETTEXT_AVAILABLE = false
 end
 
+
+
 #------------------------------------------------------------
 # the top-level module
 #
@@ -53,34 +86,14 @@ module Puppet
   require 'puppet/environments'
 
   class << self
-    if Puppet::GETTEXT_AVAILABLE
-      # e.g. ~/code/puppet/locales. Also when running as a gem.
-      local_locale_path = File.absolute_path('../locales', File.dirname(__FILE__))
-      # e.g. /opt/puppetlabs/puppet/share/locale
-      posix_system_locale_path = File.absolute_path('../../../share/locale', File.dirname(__FILE__))
-      # e.g. C:\Program Files\Puppet Labs\Puppet\puppet\share\locale
-      win32_system_locale_path = File.absolute_path('../../../../../puppet/share/locale', File.dirname(__FILE__))
-
-      if File.exist?(local_locale_path)
-        locale_path = local_locale_path
-      elsif File.exist?(win32_system_locale_path)
-        locale_path = win32_system_locale_path
-      elsif File.exist?(posix_system_locale_path)
-        locale_path = posix_system_locale_path
+    if Puppet::GETTEXT_AVAILABLE && Puppet::LOCALE_PATH
+      if GettextSetup.method(:initialize).parameters.count == 1
+        # Will load translations from PO files only
+        GettextSetup.initialize(Puppet::LOCALE_PATH)
       else
-        # We couldn't load our locale data.
-        locale_path = nil
-      end
-
-      if locale_path
-        if Gem.loaded_specs['gettext-setup'].version < Gem::Version.new('0.8')
-          # Will load translations from PO files only
-          GettextSetup.initialize(locale_path)
-        else
-          GettextSetup.initialize(locale_path, :file_format => :mo)
-        end
-        FastGettext.locale = GettextSetup.negotiate_locale(Locale.current.language)
+        GettextSetup.initialize(Puppet::LOCALE_PATH, :file_format => :mo)
       end
+      FastGettext.locale = GettextSetup.negotiate_locale(Locale.current.language)
     end
 
     include Puppet::Util
@@ -164,8 +177,8 @@ module Puppet
 
   # Now that settings are loaded we have the code loaded to be able to issue
   # deprecation warnings. Warn if we're on a deprecated ruby version.
-  if Gem::Version.new(RUBY_VERSION) < Gem::Version.new(Puppet::OLDEST_RECOMMENDED_RUBY_VERSION)
-    Puppet.deprecation_warning("Support for ruby version #{RUBY_VERSION} is deprecated and will be removed in a future release. See https://docs.puppet.com/puppet/latest/system_requirements.html#ruby for a list of supported ruby versions.")
+  if Gem::Version.new(RUBY_VERSION.dup) < Gem::Version.new(Puppet::OLDEST_RECOMMENDED_RUBY_VERSION)
+    Puppet.deprecation_warning(_("Support for ruby version %{version} is deprecated and will be removed in a future release. See https://docs.puppet.com/puppet/latest/system_requirements.html#ruby for a list of supported ruby versions.") % { version: RUBY_VERSION })
   end
 
   # Initialize puppet's settings. This is intended only for use by external tools that are not
@@ -212,7 +225,7 @@ module Puppet
   # code was deprecated in 2008, but this is still in heavy use.  I suppose
   # this can count as a soft deprecation for the next dev. --daniel 2011-04-12
   def self.newtype(name, options = {}, &block)
-    Puppet.deprecation_warning("Creating #{name} via Puppet.newtype is deprecated and will be removed in a future release. Use Puppet::Type.newtype instead.")
+    Puppet.deprecation_warning(_("Creating %{name} via Puppet.newtype is deprecated and will be removed in a future release. Use Puppet::Type.newtype instead.") % { name: name })
     Puppet::Type.newtype(name, options, &block)
   end
 
@@ -231,16 +244,18 @@ module Puppet
     basemodulepath = Puppet::Node::Environment.split_path(settings[:basemodulepath])
 
     if environmentpath.nil? || environmentpath.empty?
-      raise(Puppet::Error, "The environmentpath setting cannot be empty or nil.")
+      raise(Puppet::Error, _("The environmentpath setting cannot be empty or nil."))
     else
       loaders = Puppet::Environments::Directories.from_path(environmentpath, basemodulepath)
       # in case the configured environment (used for the default sometimes)
       # doesn't exist
       default_environment = Puppet[:environment].to_sym
       if default_environment == :production
+        modulepath = settings[:modulepath]
+        modulepath = (modulepath.nil? || '' == modulepath) ? basemodulepath : Puppet::Node::Environment.split_path(modulepath)
         loaders << Puppet::Environments::StaticPrivate.new(
           Puppet::Node::Environment.create(default_environment,
-                                           basemodulepath,
+                                           modulepath,
                                            Puppet::Node::Environment::NO_MANIFEST))
       end
     end
@@ -252,6 +267,7 @@ module Puppet
         Puppet::Network::HTTP::NoCachePool.new
       },
       :ssl_host => proc { Puppet::SSL::Host.localhost },
+      :plugins => proc { Puppet::Plugins::Configuration.load_plugins }
     }
   end
 
@@ -338,4 +354,4 @@ require 'puppet/data_binding'
 require 'puppet/util/storage'
 require 'puppet/status'
 require 'puppet/file_bucket/file'
-require 'puppet/plugins'
+require 'puppet/plugins/configuration'
