@@ -21,7 +21,7 @@ class Puppet::Application::Agent < Puppet::Application
   def preinit
     # Do an initial trap, so that cancels don't get a stack trace.
     Signal.trap(:INT) do
-      $stderr.puts "Cancelling startup"
+      $stderr.puts _("Cancelling startup")
       exit(0)
     end
 
@@ -71,6 +71,10 @@ class Puppet::Application::Agent < Puppet::Application
     options[:waitforcert] = arg.to_i
   end
 
+  option("--job-id ID") do |arg|
+    options[:job_id] = arg
+  end
+
   def help
     <<-'HELP'
 
@@ -232,6 +236,10 @@ generated by running puppet agent with '--genconfig'.
 * --help:
   Print this help message
 
+* --job-id:
+  Attach the specified job id to the catalog request and the report used for
+  this agent run. This option only works when '--onetime' is used.
+
 * --logdest:
   Where to send log messages. Choose between 'syslog' (the POSIX syslog
   service), 'eventlog' (the Windows Event Log), 'console', or the path to a log
@@ -264,7 +272,7 @@ generated by running puppet agent with '--genconfig'.
 * --test:
   Enable the most common options used for testing. These are 'onetime',
   'verbose', 'no-daemonize', 'no-usecacheonfailure', 'detailed-exitcodes',
-  'no-splay', 'show_diff', and 'no-use_cached_catalog'.
+  'no-splay', and 'show_diff'.
 
 * --verbose:
   Turn on verbose reporting.
@@ -309,7 +317,7 @@ Luke Kanies
 
 COPYRIGHT
 ---------
-Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
+Copyright (c) 2011 Puppet Inc., LLC Licensed under the Apache 2.0 License
 
     HELP
   end
@@ -338,19 +346,19 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
   def fingerprint
     host = Puppet::SSL::Host.new
     unless cert = host.certificate || host.certificate_request
-      $stderr.puts "Fingerprint asked but no certificate nor certificate request have yet been issued"
+      $stderr.puts _("Fingerprint asked but no certificate nor certificate request have yet been issued")
       exit(1)
       return
     end
     unless digest = cert.digest(options[:digest].to_s)
-      raise ArgumentError, "Could not get fingerprint for digest '#{options[:digest]}'"
+      raise ArgumentError, _("Could not get fingerprint for digest '%{digest}'") % { digest: options[:digest] }
     end
     puts digest.to_s
   end
 
   def onetime(daemon)
     begin
-      exitstatus = daemon.agent.run
+      exitstatus = daemon.agent.run(:job_id => options[:job_id])
     rescue => detail
       Puppet.log_exception(detail)
     end
@@ -367,7 +375,7 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
   end
 
   def main(daemon)
-    Puppet.notice "Starting Puppet client version #{Puppet.version}"
+    Puppet.notice _("Starting Puppet client version %{version}") % { version: Puppet.version }
     daemon.start
   end
 
@@ -377,14 +385,13 @@ Copyright (c) 2011 Puppet Labs, LLC Licensed under the Apache 2.0 License
     Puppet.settings.handlearg("--no-splay")
     Puppet.settings.handlearg("--show_diff")
     Puppet.settings.handlearg("--no-daemonize")
-    Puppet.settings.handlearg("--no-use_cached_catalog")
     options[:verbose] = true
     Puppet[:onetime] = true
     options[:detailed_exitcodes] = true
   end
 
   def setup
-    raise ArgumentError, "The puppet agent command does not take parameters" unless command_line.args.empty?
+    raise ArgumentError, _("The puppet agent command does not take parameters") unless command_line.args.empty?
 
     setup_test if options[:test]
 
