@@ -1,6 +1,6 @@
 require 'puppet/util/logging'
-require 'semver'
 require 'json'
+require 'semantic_puppet/gem_version'
 
 # Support for modules
 class Puppet::Module
@@ -53,15 +53,42 @@ class Puppet::Module
     return false
   end
 
+  # @api private
+  def self.parse_range(range, strict)
+    @parse_range_method ||= SemanticPuppet::VersionRange.method(:parse)
+    if @parse_range_method.arity == 1
+      @semver_gem_version ||= SemanticPuppet::Version.parse(SemanticPuppet::VERSION)
+
+      # Give user a heads-up if the desired strict setting cannot be honored
+      if strict
+        if @semver_gem_version.major < 1
+          Puppet.warn_once('strict_version_ranges', 'version_range_cannot_be_strict',
+            _('VersionRanges will never be strict when using non-vendored SemanticPuppet gem, version %{version}') % { version: @semver_gem_version},
+            :default, :default, :notice)
+        end
+      else
+        if @semver_gem_version.major >= 1
+          Puppet.warn_once('strict_version_ranges', 'version_range_always_strict',
+            _('VersionRanges will always be strict when using non-vendored SemanticPuppet gem, version %{version}') % { version: @semver_gem_version},
+            :default, :default, :notice)
+        end
+      end
+      @parse_range_method.call(range)
+    else
+      @parse_range_method.call(range, strict)
+    end
+  end
+
   attr_reader :name, :environment, :path, :metadata
   attr_writer :environment
 
   attr_accessor :dependencies, :forge_name
   attr_accessor :source, :author, :version, :license, :summary, :description, :project_page
 
-  def initialize(name, path, environment)
+  def initialize(name, path, environment, strict_semver = true)
     @name = name
     @path = path
+    @strict_semver = strict_semver
     @environment = environment
 
     assert_validity
@@ -96,7 +123,7 @@ class Puppet::Module
   FILETYPES.each do |type, location|
     # A boolean method to let external callers determine if
     # we have files of a given type.
-    define_method(type +'?') do
+    define_method(type + '?') do
       type_subpath = subpath(location)
       unless Puppet::FileSystem.exist?(type_subpath)
         Puppet.debug("No #{type} found in subpath '#{type_subpath}' " +
@@ -146,7 +173,9 @@ class Puppet::Module
   rescue JSON::JSONError => e
     msg = "#{name} has an invalid and unparsable metadata.json file. The parse error: #{e.message}"
     case Puppet[:strict]
-    when :off, :warning
+    when :off
+      Puppet.debug(msg)
+    when :warning
       Puppet.warning(msg)
     when :error
       raise FaultyMetadata, msg
@@ -326,9 +355,8 @@ class Puppet::Module
 
       if version_string
         begin
-          # Suppress deprecation warnings from SemVer in 4.9. In 5.0, this will be SemanticPuppet instead
-          required_version_semver_range = SemVer[version_string, true]
-          actual_version_semver = SemVer.new(dep_mod.version, true)
+          required_version_semver_range = self.class.parse_range(version_string, @strict_semver)
+          actual_version_semver = SemanticPuppet::Version.parse(dep_mod.version)
         rescue ArgumentError
           error_details[:reason] = :non_semantic_version
           unmet_dependencies << error_details
@@ -353,6 +381,10 @@ class Puppet::Module
     self.environment == other.environment
   end
 
+  def strict_semver?
+    @strict_semver
+  end
+
   private
 
   def wanted_manifests_from(pattern)
