@@ -14,7 +14,13 @@ module Puppet
     EMPTY_ARRAY = [].freeze
     EMPTY_STRING = ''.freeze
 
-    require 'semantic'
+    MAX_INTEGER =  0x7fffffffffffffff
+    MIN_INTEGER = -0x8000000000000000
+
+    DOUBLE_COLON = '::'.freeze
+    USCORE = '_'.freeze
+
+    require 'semantic_puppet'
 
     require 'puppet/pops/patterns'
     require 'puppet/pops/utils'
@@ -26,28 +32,24 @@ module Puppet
     require 'puppet/pops/visitable'
     require 'puppet/pops/visitor'
 
-    require 'puppet/pops/containment'
-
     require 'puppet/pops/issues'
     require 'puppet/pops/semantic_error'
     require 'puppet/pops/label_provider'
     require 'puppet/pops/validation'
     require 'puppet/pops/issue_reporter'
-    require 'puppet/pops/lookup'
-    require 'puppet/pops/lookup/interpolation'
-    require 'puppet/pops/lookup/invocation'
-    require 'puppet/pops/lookup/sub_lookup'
-    require 'puppet/pops/lookup/explainer'
 
-    require 'puppet/pops/model/model'
+    require 'puppet/pops/time/timespan'
+    require 'puppet/pops/time/timestamp'
 
     # (the Types module initializes itself)
     require 'puppet/pops/types/types'
     require 'puppet/pops/types/string_converter'
+    require 'puppet/pops/lookup'
 
     require 'puppet/pops/merge_strategy'
 
     module Model
+      require 'puppet/pops/model/ast'
       require 'puppet/pops/model/tree_dumper'
       require 'puppet/pops/model/ast_transformer'
       require 'puppet/pops/model/factory'
@@ -55,37 +57,6 @@ module Puppet
       require 'puppet/pops/model/model_label_provider'
     end
 
-    module Binder
-      module SchemeHandler
-        # the handlers are auto loaded via bindings
-      end
-      module Producers
-        require 'puppet/pops/binder/producers'
-      end
-
-      require 'puppet/pops/binder/binder'
-      require 'puppet/pops/binder/bindings_model'
-      require 'puppet/pops/binder/binder_issues'
-      require 'puppet/pops/binder/bindings_checker'
-      require 'puppet/pops/binder/bindings_factory'
-      require 'puppet/pops/binder/bindings_label_provider'
-      require 'puppet/pops/binder/bindings_validator_factory'
-      require 'puppet/pops/binder/injector_entry'
-      require 'puppet/pops/binder/key_factory'
-      require 'puppet/pops/binder/injector'
-      require 'puppet/pops/binder/bindings_composer'
-      require 'puppet/pops/binder/bindings_model_dumper'
-      require 'puppet/pops/binder/system_bindings'
-      require 'puppet/pops/binder/bindings_loader'
-
-      module Config
-        require 'puppet/pops/binder/config/binder_config'
-        require 'puppet/pops/binder/config/binder_config_checker'
-        require 'puppet/pops/binder/config/issues'
-        require 'puppet/pops/binder/config/diagnostic_producer'
-      end
-    end
-
     module Resource
       require 'puppet/pops/resource/resource_type_impl'
     end
@@ -144,6 +115,6 @@ module Puppet
   require 'puppet/parser/ast/pops_bridge'
   require 'puppet/bindings'
   require 'puppet/functions'
-  require 'puppet/loaders'
+
+  Puppet::Pops::Model.register_pcore_types
 end
-require 'puppet/plugins/data_providers'
