@@ -40,7 +40,19 @@ module Puppet::Pops::LabelProvider
     count == 1 ? text : "#{text}s"
   end
 
-  private
+  # Combines several strings using commas and a final conjunction
+  def combine_strings(strings, conjunction = 'or')
+    case strings.size
+    when 0
+      ''
+    when 1
+      strings[0]
+    when 2
+      "#{strings[0]} #{conjunction} #{strings[1]}"
+    else
+      "#{strings[0...-1].join(', ')}, #{conjunction} #{strings.last}"
+    end
+  end
 
   # Produces an *indefinite article* (a/an) for the given text ('a' if
   # it starts with a vowel) This is obviously flawed in the general
@@ -52,6 +64,8 @@ module Puppet::Pops::LabelProvider
     article_for_letter(first_letter_of(s))
   end
 
+  private
+
   def first_letter_of(string)
     char = string[0,1]
     if SKIPPED_CHARACTERS.include? char
