@@ -10,7 +10,7 @@
 # This implementation does not support
 # * setting 'strict' - strictness (must refer to an existing type) is always true
 # * does not support the indirector
-# 
+#
 #
 module Puppet::Pops
 module Resource
@@ -43,7 +43,11 @@ class Param
     @name_var = name_var
   end
 
-  def self._ptype
+  def to_s
+    name
+  end
+
+  def self._pcore_type
     @ptype
   end
 end
