@@ -21,7 +21,7 @@ class ResourceTypeImpl
   include Puppet::CompilableResourceType
 
   # Returns the Puppet Type for this instance.
-  def self._ptype
+  def self._pcore_type
     @ptype
   end
 
@@ -47,6 +47,10 @@ class ResourceTypeImpl
           Types::KEY_TYPE => Types::PBooleanType::DEFAULT,
           Types::KEY_VALUE => true
         },
+        'capability' => {
+          Types::KEY_TYPE => Types::PBooleanType::DEFAULT,
+          Types::KEY_VALUE => false
+        },
       },
       EMPTY_HASH,
       [Types::KEY_NAME]
@@ -96,14 +100,14 @@ class ResourceTypeImpl
   attr_reader :parameters
   attr_reader :title_patterns_hash
   attr_reader :title_patterns
-  attr_reader :isomorphic
 
-  def initialize(name, properties = EMPTY_ARRAY, parameters = EMPTY_ARRAY, title_patterns_hash = nil, isomorphic = true)
+  def initialize(name, properties = EMPTY_ARRAY, parameters = EMPTY_ARRAY, title_patterns_hash = nil, isomorphic = true, capability = false)
     @name = name
     @properties = properties
     @parameters = parameters
     @title_patterns_hash = title_patterns_hash
     @isomorphic = isomorphic
+    @capability = capability
 
     # Compute attributes hash
     # Compute key_names (possibly compound key if there are multiple name vars).
@@ -151,7 +155,7 @@ class ResourceTypeImpl
           # TechDebt: The case of having one namevar and an empty title patterns is unspecified behavior in puppet.
           # Here, it may lead to an empty map which may or may not trigger the wanted/expected behavior.
           #
-          @title_patterns_hash.map {|k,v| [ k, [ v.map {|n| n.to_sym } ] ] }
+          @title_patterns_hash.map { |k,v| [ k, v.map { |n| [ n.to_sym ] } ] }
         end
       else
         if @title_patterns_hash.nil? || @title_patterns_hash.empty?
@@ -162,7 +166,7 @@ class ResourceTypeImpl
           #
           raise Puppet::DevError,"you must specify title patterns when there are two or more key attributes"
         end
-        @title_patterns_hash.nil? ? [] : @title_patterns_hash.map {|k,v| [ k, [ v.map {|n| n.to_sym } ] ] }
+        @title_patterns_hash.nil? ? [] : @title_patterns_hash.map { |k,v| [ k, v.map { |n| [ n.to_sym] } ] }
       end
   end
 
@@ -172,7 +176,7 @@ class ResourceTypeImpl
   end
 
   def is_capability?
-    false
+    @capability
   end
 
   # Answers if the parameter name is a parameter/attribute of this type
@@ -259,7 +263,7 @@ class ResourceTypeImpl
 
   # Answers :property, :param or :meta depending on the type of the attribute
   # According to original version, this is called millions of times
-  # and a cache is required. 
+  # and a cache is required.
   # @param name [Symbol]
   def attrtype(name)
     raise NotImplementedError, "attrtype() - returns the kind (:meta, :param, or :property) of the parameter"
