@@ -7,17 +7,17 @@ KEY_ANNOTATIONS = 'annotations'.freeze
 #
 # @api public
 module Annotatable
-  TYPE_ANNOTATION_KEY_TYPE = PType::DEFAULT # TBD
-  TYPE_ANNOTATION_VALUE_TYPE = PStructType::DEFAULT #TBD
-  TYPE_ANNOTATIONS = PHashType.new(TYPE_ANNOTATION_KEY_TYPE, TYPE_ANNOTATION_VALUE_TYPE)
+  TYPE_ANNOTATIONS = PHashType.new(PType.new(PTypeReferenceType.new('Annotation')), PHashType::DEFAULT)
 
   # @return [{PType => PStructType}] the map of annotations
   # @api public
-  attr_reader :annotations
+  def annotations
+    @annotations.nil? ? EMPTY_HASH : @annotations
+  end
 
   # @api private
-  def init_annotatable(i12n_hash)
-    @annotations = i12n_hash[KEY_ANNOTATIONS].freeze
+  def init_annotatable(init_hash)
+    @annotations = init_hash[KEY_ANNOTATIONS].freeze
   end
 
   # @api private
@@ -26,7 +26,7 @@ module Annotatable
   end
 
   # @api private
-  def i12n_hash
+  def _pcore_init_hash
     result = {}
     result[KEY_ANNOTATIONS] = @annotations unless @annotations.nil?
     result
