@@ -24,6 +24,26 @@ class TypeFormatter
     @singleton
   end
 
+  def expanded
+    tf = clone
+    tf.instance_variable_set(:@expanded, true)
+    tf
+  end
+
+  def indented(indent = 0, indent_width = 2)
+    tf = clone
+    tf.instance_variable_set(:@indent, indent)
+    tf.instance_variable_set(:@indent_width, indent_width)
+    tf
+  end
+
+  def ruby(ref_ctor)
+    tf = clone
+    tf.instance_variable_set(:@ruby, true)
+    tf.instance_variable_set(:@ref_ctor, ref_ctor)
+    tf
+  end
+
   # Produces a string representing the type
   # @api public
   #
@@ -84,9 +104,9 @@ class TypeFormatter
     if @ruby && t.is_a?(PAnyType)
       @ruby = false
       begin
-        @bld << @ref_ctor << "('"
-        @@string_visitor.visit_this_0(self, t)
-        @bld << "')"
+        @bld << @ref_ctor << '('
+        @@string_visitor.visit_this_0(self, TypeFormatter.new.string(t))
+        @bld << ')'
       ensure
         @ruby = true
       end
@@ -135,7 +155,7 @@ class TypeFormatter
   def string_PScalarType(_)  ; @bld << 'Scalar'  ; end
 
   # @api private
-  def string_PDataType(_)    ; @bld << 'Data'    ; end
+  def string_PScalarDataType(_)  ; @bld << 'ScalarData'  ; end
 
   # @api private
   def string_PNumericType(_) ; @bld << 'Numeric' ; end
@@ -304,11 +324,14 @@ class TypeFormatter
   end
 
   def string_PuppetObject(t)
-    @bld << t._ptype.name << '('
-    append_indented_string(t.i12n_hash, @indent || 0, @indent_width || 2, true)
-    @bld.chomp!
+    @bld << t._pcore_type.name << '('
+    if @indent
+      append_indented_string(t._pcore_init_hash, @indent, @indent_width, true)
+      @bld.chomp!
+    else
+      append_string(t._pcore_init_hash)
+    end
     @bld << ')'
-    newline if @indent
   end
 
   # @api private
@@ -326,7 +349,7 @@ class TypeFormatter
     if t.has_empty_range?
       append_array('Array') { append_strings([0, 0]) }
     else
-      append_array('Array', t == PArrayType::DATA) do
+      append_array('Array', t == PArrayType::DEFAULT) do
         append_strings([t.element_type], true)
         append_elements(range_array_part(t.size_type), true)
         chomp_list
@@ -339,7 +362,7 @@ class TypeFormatter
     if t.has_empty_range?
       append_array('Hash') { append_strings([0, 0]) }
     else
-      append_array('Hash', t == PHashType::DATA) do
+      append_array('Hash', t == PHashType::DEFAULT) do
         append_strings([t.key_type, t.value_type], true)
         append_elements(range_array_part(t.size_type), true)
         chomp_list
@@ -380,7 +403,7 @@ class TypeFormatter
 
   # @api private
   def string_PAnnotatedMember(m)
-    hash = m.i12n_hash
+    hash = m._pcore_init_hash
     if hash.size == 1
       string(m.type)
     else
@@ -398,7 +421,7 @@ class TypeFormatter
   # @api private
   def string_PTypeSetType(t)
     append_array('TypeSet') do
-      append_hash(t.i12n_hash.each, proc { |k| @bld << symbolic_key(k) }) do |k,v|
+      append_hash(t._pcore_init_hash.each, proc { |k| @bld << symbolic_key(k) }) do |k,v|
         case k
         when KEY_TYPES
           old_ts = @type_set
@@ -426,7 +449,7 @@ class TypeFormatter
   # @api private
   def string_PObjectType(t)
     if @expanded
-      append_object_hash(t.i12n_hash(@type_set.nil? || !@type_set.defines_type?(t)))
+      append_object_hash(t._pcore_init_hash(@type_set.nil? || !@type_set.defines_type?(t)))
     else
       @bld << (@type_set ? @type_set.name_for(t, t.label) : t.label)
     end
@@ -464,7 +487,7 @@ class TypeFormatter
   def format_type_alias_type(t, expand)
     if @type_set.nil?
       @bld << t.name
-      if expand
+      if expand && !Loader::StaticLoader::BUILTIN_ALIASES.include?(t.name)
         @bld << ' = '
         append_string(t.resolved_type)
       end
@@ -484,7 +507,17 @@ class TypeFormatter
 
   # @api private
   def string_Array(t)
-    append_array('') { append_strings(t) }
+    append_array('') do
+      if @indent && !is_short_array?(t)
+        @indent += 1
+        t.each { |elem| newline; append_string(elem); @bld << COMMA_SEP }
+        chomp_list
+        @indent -= 1
+        newline
+      else
+        append_strings(t)
+      end
+    end
   end
 
   # @api private
@@ -569,6 +602,21 @@ class TypeFormatter
 
   HASH_ENTRY_OP = ' => '.freeze
 
+  def is_short_array?(t)
+    t.empty? || 100 - @indent * @indent_width > t.inject(0) do |sum, elem|
+      case elem
+      when true, false, nil, Numeric, Symbol
+        sum + elem.inspect.length()
+      when String
+        sum + 2 + elem.length
+      when Hash, Array
+        sum + (elem.empty? ? 2 : 1000)
+      else
+        sum + 1000
+      end
+    end
+  end
+
   def range_array_part(t)
     t.nil? || t.unbounded? ? EMPTY_ARRAY : [t.from.nil? ? 'default' : t.from.to_s , t.to.nil? ? 'default' : t.to.to_s ]
   end
