@@ -24,10 +24,14 @@ class Puppet::Provider::NameService < Puppet::Provider
 
     def instances
       objects = []
-      listbyname do |name|
-        objects << new(:name => name, :ensure => :present)
+      begin
+        method = Puppet::Etc.method(:"get#{section}ent")
+        while ent = method.call
+          objects << new(:name => ent.name, :canonical_name => ent.canonical_name, :ensure => :present)
+        end
+      ensure
+        Puppet::Etc.send("end#{section}ent")
       end
-
       objects
     end
 
@@ -51,6 +55,7 @@ class Puppet::Provider::NameService < Puppet::Provider
     # List everything out by name.  Abstracted a bit so that it works
     # for both users and groups.
     def listbyname
+      Puppet.deprecation_warning(_("listbyname is deprecated and will be removed in a future release of Puppet. Please use `self.instances` to obtain a list of users."))
       names = []
       Puppet::Etc.send("set#{section()}ent")
       begin
@@ -93,7 +98,7 @@ class Puppet::Provider::NameService < Puppet::Provider
       name = name.intern if name.is_a? String
       if @checks.include? name
         block = @checks[name][:block]
-        raise ArgumentError, "Invalid value #{value}: #{@checks[name][:error]}" unless block.call(value)
+        raise ArgumentError, _("Invalid value %{value}: %{error}") % { value: value, error: @checks[name][:error] } unless block.call(value)
       end
     end
 
@@ -158,7 +163,7 @@ class Puppet::Provider::NameService < Puppet::Provider
 
   def create
     if exists?
-      info "already exists"
+      info _("already exists")
       # The object already exists
       return nil
     end
@@ -169,13 +174,13 @@ class Puppet::Provider::NameService < Puppet::Provider
         execute(cmd)
       end
     rescue Puppet::ExecutionFailure => detail
-      raise Puppet::Error, "Could not create #{@resource.class.name} #{@resource.name}: #{detail}", detail.backtrace
+      raise Puppet::Error, _("Could not create %{resource} %{name}: %{detail}") % { resource: @resource.class.name, name: @resource.name, detail: detail }, detail.backtrace
     end
   end
 
   def delete
     unless exists?
-      info "already absent"
+      info _("already absent")
       # the object already doesn't exist
       return nil
     end
@@ -183,7 +188,7 @@ class Puppet::Provider::NameService < Puppet::Provider
     begin
       execute(self.deletecmd)
     rescue Puppet::ExecutionFailure => detail
-      raise Puppet::Error, "Could not delete #{@resource.class.name} #{@resource.name}: #{detail}", detail.backtrace
+      raise Puppet::Error, _("Could not delete %{resource} %{name}: %{detail}") % { resource: @resource.class.name, name: @resource.name, detail: detail }, detail.backtrace
     end
   end
 
@@ -226,7 +231,7 @@ class Puppet::Provider::NameService < Puppet::Provider
     if @objectinfo.nil? or refresh == true
       @etcmethod ||= ("get" + self.class.section.to_s + "nam").intern
       begin
-        @objectinfo = Puppet::Etc.send(@etcmethod, @resource[:name])
+        @objectinfo = Puppet::Etc.send(@etcmethod, @canonical_name)
       rescue ArgumentError
         @objectinfo = nil
       end
@@ -276,6 +281,11 @@ class Puppet::Provider::NameService < Puppet::Provider
     super
     @custom_environment = {}
     @objectinfo = nil
+    if resource.is_a?(Hash) && !resource[:canonical_name].nil?
+      @canonical_name = resource[:canonical_name]
+    else
+      @canonical_name = resource[:name]
+    end
   end
 
   def set(param, value)
@@ -285,7 +295,7 @@ class Puppet::Provider::NameService < Puppet::Provider
     begin
       execute(cmd)
     rescue Puppet::ExecutionFailure => detail
-      raise Puppet::Error, "Could not set #{param} on #{@resource.class.name}[#{@resource.name}]: #{detail}", detail.backtrace
+      raise Puppet::Error, _("Could not set %{param} on %{resource}[%{name}]: %{detail}") % { param: param, resource: @resource.class.name, name: @resource.name, detail: detail }, detail.backtrace
     end
   end
 
