@@ -40,7 +40,7 @@ class Puppet::Settings::EnvironmentConf
   # without interpolation.  This is a special case for the default configured
   # environment returned by the Puppet::Environments::StaticPrivate loader.
   def self.static_for(environment, environment_timeout = 0, static_catalogs = false, rich_data = false)
-    Static.new(environment, environment_timeout, static_catalogs, 'none', rich_data)
+    Static.new(environment, environment_timeout, static_catalogs, nil, rich_data)
   end
 
   attr_reader :section, :path_to_env, :global_modulepath
@@ -138,13 +138,13 @@ class Puppet::Settings::EnvironmentConf
     section_keys = config.sections.keys
     main = config.sections[:main]
     if section_keys.size > 1
-      Puppet.warning("Invalid sections in environment.conf at '#{path_to_conf_file}'. Environment conf may not have sections. The following sections are being ignored: '#{(section_keys - [:main]).join(',')}'")
+      Puppet.warning(_("Invalid sections in environment.conf at '%{path_to_conf_file}'. Environment conf may not have sections. The following sections are being ignored: '%{sections}'") % { path_to_conf_file: path_to_conf_file, sections: (section_keys - [:main]).join(',') })
       valid = false
     end
 
     extraneous_settings = main.settings.map(&:name) - VALID_SETTINGS
     if !extraneous_settings.empty?
-      Puppet.warning("Invalid settings in environment.conf at '#{path_to_conf_file}'. The following unknown setting(s) are being ignored: #{extraneous_settings.join(', ')}")
+      Puppet.warning(_("Invalid settings in environment.conf at '%{path_to_conf_file}'. The following unknown setting(s) are being ignored: %{ignored_settings}") % { path_to_conf_file: path_to_conf_file, ignored_settings: extraneous_settings.join(', ') })
       valid = false
     end
 
@@ -176,7 +176,7 @@ class Puppet::Settings::EnvironmentConf
     attr_reader :rich_data
     attr_reader :static_catalogs
 
-    def initialize(environment, environment_timeout, static_catalogs, environment_data_provider = 'none', rich_data = false)
+    def initialize(environment, environment_timeout, static_catalogs, environment_data_provider = nil, rich_data = false)
       @environment = environment
       @environment_timeout = environment_timeout
       @static_catalogs = static_catalogs
@@ -184,6 +184,10 @@ class Puppet::Settings::EnvironmentConf
       @rich_data = rich_data
     end
 
+    def path_to_env
+      nil
+    end
+
     def manifest
       @environment.manifest
     end
