@@ -11,15 +11,15 @@ Puppet::Type.newtype(:yumrepo) do
 
     Continuation lines that yum supports (for the `baseurl`, for example)
     are not supported. This type does not attempt to read or verify the
-    exinstence of files listed in the `include` attribute."
+    existence of files listed in the `include` attribute."
 
   # Ensure yumrepos can be removed too.
   ensurable
   # Doc string for properties that can be made 'absent'
   ABSENT_DOC="Set this to `absent` to remove it from the file completely."
   # False can be false/0/no and True can be true/1/yes in yum.
-  YUM_BOOLEAN=/^(True|False|0|1|No|Yes)$/i
-  YUM_BOOLEAN_DOC="Valid values are: False/0/No or True/1/Yes."
+  YUM_BOOLEAN=/^(true|false|0|1|no|yes)$/
+  YUM_BOOLEAN_DOC="Valid values are: false/0/no or true/1/yes."
 
   # Common munge logic for YUM_BOOLEAN values. Munges for two requirements:
   # 1) Because of how regex validation works in Puppet::Parameter::Value,
@@ -62,7 +62,7 @@ Puppet::Type.newtype(:yumrepo) do
       parsed = URI.parse(value)
 
       unless VALID_SCHEMES.include?(parsed.scheme)
-        raise "Must be a valid URL"
+        raise _("Must be a valid URL")
       end
     end
   end
@@ -79,7 +79,7 @@ Puppet::Type.newtype(:yumrepo) do
         parsed = URI.parse(uri)
 
         unless VALID_SCHEMES.include?(parsed.scheme)
-          raise "Must be a valid URL"
+          raise _("Must be a valid URL")
         end
       end
     end
@@ -126,7 +126,7 @@ Puppet::Type.newtype(:yumrepo) do
         parsed = URI.parse(uri)
 
         unless VALID_SCHEMES.include?(parsed.scheme)
-          raise "Must be a valid URL"
+          raise _("Must be a valid URL")
         end
       end
     end
@@ -150,7 +150,7 @@ Puppet::Type.newtype(:yumrepo) do
       parsed = URI.parse(value)
 
       unless VALID_SCHEMES.include?(parsed.scheme)
-        raise "Must be a valid URL"
+        raise _("Must be a valid URL")
       end
     end
   end
@@ -172,7 +172,7 @@ Puppet::Type.newtype(:yumrepo) do
       parsed = URI.parse(value)
 
       unless VALID_SCHEMES.include?(parsed.scheme)
-        raise "Must be a valid URL"
+        raise _("Must be a valid URL")
       end
     end
   end
@@ -258,7 +258,7 @@ Puppet::Type.newtype(:yumrepo) do
     validate do |value|
       next if value.to_s == 'absent'
       unless (1..99).include?(value.to_i)
-        fail("Must be within range 1-99")
+        fail(_("Must be within range 1-99"))
       end
     end
   end
@@ -300,7 +300,7 @@ Puppet::Type.newtype(:yumrepo) do
       parsed = URI.parse(value)
 
       unless VALID_SCHEMES.include?(parsed.scheme)
-        raise "Must be a valid URL"
+        raise _("Must be a valid URL")
       end
     end
   end
@@ -366,7 +366,7 @@ Puppet::Type.newtype(:yumrepo) do
       parsed = URI.parse(value)
 
       unless VALID_SCHEMES.include?(parsed.scheme)
-        raise "Must be a valid URL"
+        raise _("Must be a valid URL")
       end
     end
   end
