@@ -45,7 +45,7 @@ module Puppet::Util::Plist
                                                   {:failonfail => true, :combine => true})
           return parse_plist(plist)
         rescue Puppet::ExecutionFailure => detail
-          Puppet.warning("Cannot read file #{file_path}; Puppet is skipping it.\n" + "Details: #{detail}")
+          Puppet.warning(_("Cannot read file %{file_path}; Puppet is skipping it.\n") % { file_path: file_path } + _("Details: %{detail}") % { detail: detail })
         end
       end
       return nil
@@ -54,9 +54,14 @@ module Puppet::Util::Plist
     # Read plist text using the CFPropertyList gem.
     def parse_plist(plist_data, file_path = '')
       bad_xml_doctype = /^.*<!DOCTYPE plist PUBLIC -\/\/Apple Computer.*$/
-      if plist_data =~ bad_xml_doctype
-        plist_data.gsub!( bad_xml_doctype, plist_xml_doctype )
-        Puppet.debug("Had to fix plist with incorrect DOCTYPE declaration: #{file_path}")
+      begin
+        if plist_data =~ bad_xml_doctype
+          plist_data.gsub!( bad_xml_doctype, plist_xml_doctype )
+          Puppet.debug("Had to fix plist with incorrect DOCTYPE declaration: #{file_path}")
+        end
+      rescue ArgumentError => e
+        Puppet.debug "Failed with #{e.class} on #{file_path}: #{e.inspect}"
+        return nil
       end
 
       begin
@@ -73,6 +78,12 @@ module Puppet::Util::Plist
     # stubbing purposes
     #
     # @api private
+    #
+    # @param args [String] Extra file operation mode information to use
+    #   (defaults to read-only mode 'r')
+    #   This is the standard mechanism Ruby uses in the IO class, and therefore
+    #   encoding may be explicitly like fmode : encoding or fmode : "BOM|UTF-*"
+    #   for example, a:ASCII or w+:UTF-8
     def open_file_with_args(file, args)
       File.open(file, args).read
     end
@@ -114,6 +125,8 @@ module Puppet::Util::Plist
         plist_format = CFPropertyList::List::FORMAT_XML
       elsif format.to_sym == :binary
         plist_format = CFPropertyList::List::FORMAT_BINARY
+      elsif format.to_sym == :plain
+        plist_format = CFPropertyList::List::FORMAT_PLAIN
       else
         raise FormatError.new "Unknown plist format #{format}"
       end
@@ -127,7 +140,7 @@ module Puppet::Util::Plist
         plist_to_save.value = CFPropertyList.guess(plist)
         plist_to_save.save(file_path, to_format(format), :formatted => true)
       rescue IOError => e
-        Puppet.err("Unable to write the file #{file_path}. #{e.inspect}")
+        Puppet.err(_("Unable to write the file %{file_path}. %{error}") % { file_path: file_path, error: e.inspect })
       end
     end
 
